@@ -69,7 +69,7 @@ StatEllip <- ggproto('StatEllip', Stat,
   },
   compute_panel = function(self, data, scales, n = 360) {
     if (empty_data(data)) return(data)
-    data$group <- make_unique(as.character(data$group))
+    data$group <- make_unique(data$group)
     n_ellipses <- nrow(data)
     data <- data[rep(seq_len(n_ellipses), each = n), ]
     points <- rep(seq(0, 2 * pi, length.out = n + 1)[seq_len(n)],
@@ -94,7 +94,7 @@ stat_ellip <- function(mapping = NULL, data = NULL, geom = 'circle',
   layer(
     stat = StatEllip, data = data, mapping = mapping, geom = geom,
     position = position, show.legend = show.legend, inherit.aes = inherit.aes,
-    params = list(na.rm = na.rm, n = n, ...)
+    params = list2(na.rm = na.rm, n = n, ...)
   )
 }
 
@@ -106,6 +106,6 @@ geom_ellipse <- function(mapping = NULL, data = NULL, stat = 'ellip',
   layer(
     data = data, mapping = mapping, stat = stat, geom = GeomCircle,
     position = position, show.legend = show.legend, inherit.aes = inherit.aes,
-    params = list(n = n, na.rm = na.rm, ...)
+    params = list2(n = n, na.rm = na.rm, ...)
   )
 }
